# Reduce then number of parallel building processes to avoid oom-killer
%global _smp_ncpus_max 4

#global rc_ver		2
%global baserelease	3
%global flang_srcdir	flang-%{version}%{?rc_ver:rc%{rc_ver}}.src
%global maj_ver		11
%global min_ver		0
%global patch_ver	1

%define major %{maj_ver}.%{min_ver}
%define minor 1

%define libname %mklibname flang %{major}
%define devname %mklibname -d flang

Name:		flang
Version:	%{maj_ver}.%{min_ver}.%{patch_ver}
Release:	%mkrel %{baserelease}%{?rc_ver:.rc%{rc_ver}}
Summary:	A Fortran language front-end designed for integration with LLVM
Group:		Development/Other
License:	ASL 2.0 with exceptions
URL:		https://github.com/llvm/llvm-project/tree/master/flang
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}%{?rc_ver:-rc%{rc_ver}}/%{flang_srcdir}.tar.xz
Source1:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}%{?rc_ver:-rc%{rc_ver}}/%{flang_srcdir}.tar.xz.sig
Source2:	https://github.com/llvm/llvm-project/releases/download/llvmorg-9.0.1/tstellar-gpg-key.asc

# Otherwise fails when compiling with _GLIBCXX_DEBUG is defined.
Patch0:		token-sequence.patch
# Needed to compile in shared libs mode.
Patch1:		shared.patch
# Needed for llvm-googletest compatibility
Patch2:		gtest.patch
# Needed for documentation generation
Patch3:		sphinx_markdown_tables.patch

# Exclude armv7 because there isn't enough virtual memory to build on armv7 ("caught: virtual memory exhausted: Cannot allocate memory").
# Exclude aarch64 too for now
ExcludeArch: %{arm} aarch64

# Avoid gcc reaching 4GB of memory
%ifarch %{ix86} aarch64 %{arm}
%global _lto_cflags %{nil}
%global _smp_mflags -j1
%endif

BuildRequires:	gcc
BuildRequires:	gcc-c++
BuildRequires:	cmake
BuildRequires:	zlib-devel
BuildRequires:	llvm-devel = %{version}
BuildRequires:	llvm-test = %{version}
BuildRequires:	llvm-googletest = %{version}
BuildRequires:	mlir-devel = %{version}
BuildRequires:	ninja-build
BuildRequires:	python3-lit
BuildRequires:	python3-sphinx
BuildRequires:	python3-recommonmark
# For origin certification
BuildRequires:	gnupg2

# flang is not yet able to compile a running binary, thus it requires
# an external fortran compiler, set in F18_FC env var, which by default
# is set to "gfortran".
Requires:	gcc-gfortran

%description
Flang is a ground-up implementation of a Fortran front end written in modern
C++.

%package -n %libname
Summary:	Runtime libraries for flang

%description -n %libname
This package contains the runtime libraries for flang.


%package -n %devname
Summary:	Development header files for flang
Provides:	flang-devel = %version-%release
Requires:	%{libname} = %{version}-%{release}
	
%description -n %devname
Flang header files.

%package doc
Summary:	Documentation for Flang
BuildArch:	noarch
Requires:	%{name} = %{version}-%{release}

%description doc
Documentation for Flang

%prep
%{gpgverify} --keyring='%{SOURCE2}' --signature='%{SOURCE1}' --data='%{SOURCE0}'
%autosetup -n %{flang_srcdir} -p1

%build
%cmake -GNinja \
	-DMLIR_TABLEGEN_EXE=%{_bindir}/mlir-tblgen \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DCMAKE_INSTALL_RPATH=";" \
	-DLLVM_MAIN_SRC_DIR=%{_datadir}/llvm/src \
	-DBUILD_SHARED_LIBS:BOOL=ON \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_EXTERNAL_LIT=%{_bindir}/lit \
	-DCMAKE_PREFIX_PATH=%{_libdir}/cmake/llvm/ \
\
	-DFLANG_INCLUDE_DOCS:BOOL=ON \
	-DLLVM_ENABLE_SPHINX:BOOL=ON \
	-DSPHINX_WARNINGS_AS_ERRORS=OFF \
	-DSPHINX_EXECUTABLE=%{_bindir}/sphinx-build-3 \
\
%if 0%{?__isa_bits} == 64
	-DLLVM_LIBDIR_SUFFIX=64
%else
	-DLLVM_LIBDIR_SUFFIX=
%endif

# Avoid gcc reaching 4GB of memory
%ifarch %{ix86}
sed -i -e 's/-g /-g1 /g' -e 's/-O2/-O1/g' %{_builddir}/%{flang_srcdir}/%{_vpath_builddir}/build.ninja
%endif

export LD_LIBRARY_PATH=%{_builddir}/%{flang_srcdir}/%{_vpath_builddir}/lib
%cmake_build
%cmake_build --target docs-flang-html

%install
%cmake_install
# this is a test binary
rm -f %{buildroot}%{_bindir}/f18-parse-demo

install -d %{buildroot}%{_pkgdocdir}/html
cp -r %{_vpath_builddir}/docs/html/* %{buildroot}%{_pkgdocdir}/html/

chmod 0755 %{buildroot}%{_bindir}/flang

%check
# these tests fail on all or specific arches
rm test/Semantics/resolve63.f90
%ifarch %{ix86}
rm -f test/Fir/fir-ops.fir
rm -f test/Semantics/assign03.f90
rm -f test/Semantics/data05.f90
rm -f test/Semantics/offsets01.f90
rm -f test/Semantics/offsets02.f90
%endif
export LD_LIBRARY_PATH=$(pwd)/${_vpath_builddir}/lib
%cmake_build --target check-flang 

%files
%license LICENSE.txt
%{_bindir}/f18
%{_bindir}/tco
%{_bindir}/flang

%files -n %libname
%{_libdir}/libFortranLower.so.%{maj_ver}{,.*}
%{_libdir}/libFIROptimizer.so.%{maj_ver}{,.*}
%{_libdir}/libFortranSemantics.so.%{maj_ver}{,.*}
%{_libdir}/libFortranCommon.so.%{maj_ver}{,.*}
%{_libdir}/libFortranRuntime.so.%{maj_ver}{,.*}
%{_libdir}/libFortranDecimal.so.%{maj_ver}{,.*}
%{_libdir}/libFortranEvaluate.so.%{maj_ver}{,.*}
%{_libdir}/libFortranParser.so.%{maj_ver}{,.*}

%files -n %devname
%{_libdir}/libFortranLower.so
%{_libdir}/libFortranParser.so
%{_libdir}/libFortranCommon.so
%{_libdir}/libFortranSemantics.so
%{_libdir}/libFIROptimizer.so
%{_libdir}/libFortranDecimal.so
%{_libdir}/libFortranRuntime.so
%{_libdir}/libFortranEvaluate.so
%{_includedir}/flang
%{_libdir}/cmake/*

%files doc
%dir %{_pkgdocdir}
%doc %{_pkgdocdir}/html/



%changelog
* Thu Jan 07 2021 ghibo <ghibo> 11.0.1-3.mga8
+ Revision: 1669684
- Reduce _smp_ncpus_max to avoid oom-killer
+ tv <tv>
- LLVM 11.0.1 final

* Thu Dec 31 2020 ghibo <ghibo> 11.0.1-2.rc2.mga8
+ Revision: 1667288
- Update to release 11.0.1rc2

* Fri Dec 04 2020 ghibo <ghibo> 11.0.1-2.rc1.mga8
+ Revision: 1652442
- Add gfortran in Requires
- Reduce amount of parallel building for aarch64 and arm
- Exclude building on arm and aarch64
- imported package flang


* Wed Dec 02 2020 ghibo <ghibo> 11.0.1-rc1-1.mga8
- initial release, synched from FC.
k